######################################################################
## Example 1 in Zeng, He, and Zhu (2010)
######################################################################

rm(list = ls())
source("sim-lasso.R")
dyn.load("sim-lasso-clib.dll")
## dyn.load("sim-lasso-clib.so")   ### if Unix/Linux or MacOS

n = 100; p = 20
b = c(1, -1, 2, -0.5, rep(0, 16))
yfun = expression( x%*%b + 4 * sqrt(abs(x%*%b + 1)) + e )

x = matrix(runif(n * p, -2, 2), ncol = p)
e = rnorm(n)
y = eval(yfun)

h0 = 1.06 * median(sd(x)) * length(y)^(-1/5)
lda.seq = seq(0, 1.0, by = 0.01)

fit = sim.lasso.path(y, x, h0, lda.seq)

### figure 1(a)

cutindex = (rowSums(fit$beta^2) != 0)
matplot(lda.seq[cutindex], fit$beta[cutindex, ], xlim = c(0, 1),
        type = "l", xlab = "", ylab = "")
mtext("labmda", side = 1, line = 2.5)
mtext("theta", side = 2, line = 2.5)

### figure 1(b)

index = (b != 0)
trupos = numeric(length(lda.seq))
falpos = numeric(length(lda.seq))
for(i in 1:length(lda.seq))
{
  trupos[i] = mean(fit$beta[i, index] != 0)
  falpos[i] = mean(fit$beta[i, !index] != 0)
}

cutindex = ((trupos + falpos) != 0)

plot(range(lda.seq), range(trupos, falpos), type = "n", xlab = "", ylab = "")
mtext("lambda", side = 1, line = 2.5)
mtext("true / false positive rate", side = 2, line = 2.5)
lines(lda.seq[cutindex], trupos[cutindex], type = "l", lty = 1, col = "blue")
lines(lda.seq[cutindex], falpos[cutindex], type = "l", lty = 2)
text(0.6, 1.0, "true positive rate", pos = 1)
text(0.6, 0.07, "false positive rate", pos = 3)

### figure 1(c)

angles = numeric(length(lda.seq))
for(i in 1:length(lda.seq)) angles[i] = dist.vec(fit$beta[i, ], b)

plot(lda.seq, angles, type = "l", xlab = "", ylab = "")
mtext("labmda", side = 1, line = 2.5)
mtext("angle (in degree)", side = 2, line = 2.5)

### figure 1(d)

plot(lda.seq, fit$bj[, 1], type = "l", lty = 2, xlab = "", ylab = "")
text(0.9, 0, "ell-1 norm", pos = 3)
par(new = T)
plot(lda.seq, fit$bj[, 2] / n, type = "l", col = "blue", axes = F, xlab = "", ylab = "")
text(0.9, 1.0, "% of bj = 0", pos = 1)
axis(4, pretty(range(fit$bj[, 2] / n), 5))
mtext("percentage of bj = 0", side = 4, line = 2.5)
mtext("lambda", side = 1, line = 2.5)
mtext("ell-1 norm of b", side = 2, line = 2.5)

######################################################################
## END OF THE FILE
######################################################################
