######################################################################
## Second part of Example 2 in Zeng, He, and Zhu (2010)
######################################################################

rm(list = ls())
source("sim-lasso.R")
dyn.load("sim-lasso-clib.dll")
## dyn.load("sim-lasso-clib.so")   ### if Unix/Linux or MacOS

######################################################################

sim.type = function(yfun, filename)
{
  n = 100; p = 20; iter = 200
  b = c(1, -1, 2, -0.5, rep(0, 16))
  lda.seq = seq(0, 1.0, by = 0.01)

  h.mat = numeric(iter)
  lambda.mat = matrix(0, nrow = iter, ncol = 2)
  colnames(lambda.mat) = c("opt", "upper")

  beta.mat = array(0, dim = c(iter, 3, p))
  angle.mat = array(0, dim = c(iter, 3))
  trupos = array(0, dim = c(iter, 3))
  falpos = array(0, dim = c(iter, 3))

  index0 = (b == 0)

  time1 = date()
  for(i in 1:iter)
  {
    cat(filename, "<===> i =", i, "<===>", date(), "\n")

    x = matrix(runif(n * p, -2, 2), ncol = p)
    e = rnorm(n)
    y = eval(yfun)

    h = n^(-1/5) * 1.06 * median( sd(x) )
    ans = choose.lambda(y, x, h, lda.seq, m = 10)
    lambda.mat[i, 1] = ans$opt
    lambda.mat[i, 2] = ans$upper
    h.mat[i] = h

    ans1 = sim.lasso(y, x, h, 0)
    beta.mat[i, 1, ] = ans1$beta
    angle.mat[i, 1] = dist.vec(ans1$beta, b)
    trupos[i, 1] = sum(ans1$beta[!index0] != 0.0)
    falpos[i, 1] = sum(ans1$beta[index0] != 0.0)

    ans2 = sim.lasso(y, x, h, lambda.mat[i, 1])
    beta.mat[i, 2, ] = ans2$beta
    angle.mat[i, 2] = dist.vec(ans2$beta, b)
    trupos[i, 2] = sum(ans2$beta[!index0] != 0.0)
    falpos[i, 2] = sum(ans2$beta[index0] != 0.0)

    ans3 = sim.lasso(y, x, h, lambda.mat[i, 2])
    beta.mat[i, 3, ] = ans3$beta
    angle.mat[i, 3] = dist.vec(ans3$beta, b)
    trupos[i, 3] = sum(ans3$beta[!index0] != 0.0)
    falpos[i, 3] = sum(ans3$beta[index0] != 0.0)
  }
  time2 = date()

  dump(c("yfun", "n", "p", "iter", "time1", "time2", "b",
         "h.mat", "lda.seq", "lambda.mat", 
         "beta.mat", "angle.mat", "trupos", "falpos"),
       file = paste(filename, ".Rdata", sep = ""))
}

######################################################################

yfun1= expression( x%*%b + 4 * sqrt(abs(x%*%b + 1)) + e )
yfun2 = expression( 2 * (x%*%b) + 10 * exp(-(x%*%b)^2 / 5) + e)
yfun3 = expression( 2 * (x%*%b + 3) * log(3 * abs(x%*%b) + 1) + e)
yfun4 = expression( 0.25 * (x%*%b)^2 + (x%*%b) + e )

sim.type(yfun1, "example-2-b-1")
sim.type(yfun2, "example-2-b-2")
sim.type(yfun3, "example-2-b-3")
sim.type(yfun4, "example-2-b-4")

cbind(colMeans(angle.mat), sd(angle.mat), colMeans(falpos) / 16, colMeans(trupos) / 4 )

######################################################################
## END OF THE FILE
######################################################################
