######################################################################
## Second part of Example 2 in Zeng, He, and Zhu (2010)
######################################################################

rm(list = ls())
source("sim-lasso.R")
dyn.load("sim-lasso-clib.dll")
## dyn.load("sim-lasso-clib.so")   ### if Unix/Linux or MacOS

compare.performance = function(yfun, filename, h0, lambda0, h1.seq, lda1.seq, h2.seq, lda2.seq)
{
  n = 100; p = 20; iter = 500
  b = c(1, -1, 2, -0.5, rep(0, 16))

  HH1 = length(h1.seq); KK1 = length(lda1.seq)
  HH2 = length(h2.seq); KK2 = length(lda2.seq)

  angle1.mat = array(0, dim = c(HH1, KK1, iter))
  angle2.mat = array(0, dim = c(HH2, KK2, iter))

  b0.mat = matrix(0, nrow = iter, ncol = p)

  time1 = date()
  for(i in 1:iter)
  {
    cat(filename, "<===> i =", i, "<===>", date(), "\n")

    x = matrix(runif(n * p, -2, 2), ncol = p)
    e = rnorm(n)
    y = eval(yfun)

    ans = OPG.lasso(y, x, h0, lambda0)
    b0.mat[i, ] = ans$beta 

    for(j in 1:HH1)
    for(k in 1:KK1)
    {
      ans = sim.lasso(y, x, h1.seq[j], lda1.seq[k], b0 = b0.mat[i, ])
      angle1.mat[j, k, i] = dist.vec(ans$beta, b)
    }

    for(j in 1:HH2)
    for(k in 1:KK2)
    {
      ans = sim.sMAVE(y, x, h2.seq[j], lda2.seq[k], b0 = b0.mat[i, ])
      angle2.mat[j, k, i] = dist.vec(ans$beta, b)
    }
  }
  time2 = date()

  dump(c("yfun", "n", "p", "iter", "time1", "time2", "b",
         "h0", "lambda0", "h1.seq", "lda1.seq", "h2.seq", "lda2.seq", 
         "angle1.mat", "angle2.mat", "b0.mat"),
       file = paste(filename, ".Rdata", sep = ""))
}


######################################################################

bowlfun = function(x, a = 0, b = 1)
{
  y = rep(abs(a * b), length(x))
  index1 = (x > abs(a))
  y[index1] = x[index1] * abs(b)
  index2 = (x < -abs(a))
  y[index2] = -x[index2] * abs(b)
  y
}


######################################################################


yfun1a = expression( x%*%b + 4 * sqrt(abs(x%*%b + 1)) +  e )
yfun2a = expression( 2 * (x%*%b) + 10 * exp(-(x%*%b)^2 / 5) + e)
yfun3a = expression( 2 * (x%*%b + 3) * log(3 * abs(x%*%b) + 1) + e)
yfun4a = expression( 0.25 * (x%*%b)^2 + (x%*%b) + e )
yfun5a = expression( 0.5 * (x%*%b)^2 + e )
yfun6a = expression( 9 + 3 * bowlfun(x%*%b, 3, 3) + 0.3 * e )


yfun1b = expression( x%*%b + 4 * sqrt(abs(x%*%b + 1)) +  3 * e )
yfun2b = expression( 2 * (x%*%b) + 10 * exp(-(x%*%b)^2 / 5) + 3 * e)
yfun3b = expression( 2 * (x%*%b + 3) * log(3 * abs(x%*%b) + 1) + 3 * e)
yfun4b = expression( 0.25 * (x%*%b)^2 + (x%*%b) + 3 * e )
yfun5b = expression( 0.5 * (x%*%b)^2 + 3 * e )
yfun6b = expression( 9 + 3 * bowlfun(x%*%b, 3, 3) + 0.9 * e )


######################################################################

h1.seq   = seq(0.29, 0.39, by = 0.01)
lda1.seq = seq(0.11, 0.21, by = 0.01)
h2.seq   = seq(0.30, 0.40, by = 0.01)
lda2.seq = seq(53, 63, by = 1)
compare.performance(yfun1a, "ex-3-1a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.24, 0.34, by = 0.01)
lda1.seq = seq(0.10, 0.20, by = 0.01)
h2.seq   = seq(0.22, 0.32, by = 0.01)
lda2.seq = seq(87, 97, by = 1)
compare.performance(yfun2a, "ex-3-2a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.22, 0.32, by = 0.01)
lda1.seq = seq(0.14, 0.24, by = 0.01)
h2.seq   = seq(0.23, 0.33, by = 0.01)
lda2.seq = seq(212, 222, by = 1)
compare.performance(yfun3a, "ex-3-3a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.48, 0.58, by = 0.01)
lda1.seq = seq(0.12, 0.22, by = 0.01)
h2.seq   = seq(0.44, 0.54, by = 0.01)
lda2.seq = seq(46, 56, by = 1)
compare.performance(yfun4a, "ex-3-4a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.33, 0.48, by = 0.01)
lda1.seq = seq(0.13, 0.23, by = 0.01)
h2.seq   = seq(0.38, 0.48, by = 0.01)
lda2.seq = seq(95, 105, by = 1)
compare.performance(yfun5a, "ex-3-5a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.23, 0.33, by = 0.01)
lda1.seq = seq(0.03, 0.13, by = 0.01)
h2.seq   = seq(0.35, 0.45, by = 0.01)
lda2.seq = seq(18, 28, by = 1)
compare.performance(yfun6a, "ex-3-6a", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################
######################################################################

h1.seq   = seq(0.55, 0.65, by = 0.01)
lda1.seq = seq(0.42, 0.52, by = 0.01)
h2.seq   = seq(0.56, 0.66, by = 0.01)
lda2.seq = seq(144, 154, by = 1)
compare.performance(yfun1b, "ex-3-1b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.41, 0.51, by = 0.01)
lda1.seq = seq(0.49, 0.59, by = 0.01)
h2.seq   = seq(0.41, 0.51, by = 0.01)
lda2.seq = seq(225, 235, by = 1)
compare.performance(yfun2b, "ex-3-2b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.38, 0.48, by = 0.01)
lda1.seq = seq(0.49, 0.59, by = 0.01)
h2.seq   = seq(0.35, 0.45, by = 0.01)
lda2.seq = seq(683, 693, by = 1)
compare.performance(yfun3b, "ex-3-3b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.79, 0.89, by = 0.01)
lda1.seq = seq(0.42, 0.52, by = 0.01)
h2.seq   = seq(0.69, 0.79, by = 0.01)
lda2.seq = seq(116, 126, by = 1)
compare.performance(yfun4b, "ex-3-4b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.58, 0.68, by = 0.01)
lda1.seq = seq(0.40, 0.50, by = 0.01)
h2.seq   = seq(0.53, 0.63, by = 0.01)
lda2.seq = seq(188, 198, by = 1)
compare.performance(yfun5b, "ex-3-5b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################

h1.seq   = seq(0.25, 0.30, by = 0.01)
lda1.seq = seq(0.05, 0.15, by = 0.01)
h2.seq   = seq(0.34, 0.44, by = 0.01)
lda2.seq = seq(15, 25, by = 1)
compare.performance(yfun6b, "ex-3-6b", 1.5, 0.8, h1.seq, lda1.seq, h2.seq, lda2.seq)

######################################################################
## END OF THE FILE
######################################################################
