######################################################################
## Example 4 in Zeng, He, and Zhu (2010)
######################################################################

rm(list = ls())
source("sim-lasso.R")
dyn.load("sim-lasso-clib.dll")
## dyn.load("sim-lasso-clib.so")   ### if Unix/Linux or MacOS

ozone = read.table("ozone.txt", h = T)

y = ozone$OZONE
x1 = ozone$RADIATIO
x2 = ozone$TEMPERAT
x3 = ozone$WIND
x4 = x1 * x1
x5 = x2 * x2
x6 = x3 * x3
x7 = x1 * x2
x8 = x1 * x3
x9 = x2 * x3
xraw = cbind(x1, x2, x3, x4, x5, x6, x7, x8, x9)
x = scale(xraw)

h0 = 1.06 * median(sd(x)) * length(y)^(-1/5)
lda.seq = seq(0, 0.02, length = 51)
fit = sim.lasso.path(y, x, h0, lda.seq) 

niter = 10000

######################################################################

pardata = function(npop, nsample, m = 2)
{
  selected = matrix(0, nrow = m, ncol = nsample)
  for(i in 1:m)
    selected[i, ] = sort(sample(1:npop, nsample))
  selected
}

selected = pardata(NROW(x), 56, niter)

######################################################################

check.pred = function(y, x, selected)
{
  n = NROW(x); m = NROW(selected)
  h0.mat = numeric(m)
  errormat = numeric(m)
  for(i in 1:m)
  {
    index1 = selected[i, ]
    index2 = (1:n)[-index1]

    h0 = 1.06 * median(sd(x[index1, ])) * length(y[index1])^(-1/5)
    fit = sim(y[index1], x[index1, ], h0)
    yhat = prediction(y[index1], x[index1, ], h0, fit$beta, x[index2, ])
    errormat[i] = mean( (y[index2] - yhat$yhat)^2 )
    h0.mat[i] = h0
  }
  list(errormat = errormat, h0.mat = h0.mat,
       mean = mean(errormat), sd = sd(errormat))
}

ans1 = check.pred(y, x[, c(1, 2, 3)], selected)
ans2 = check.pred(y, x, selected)
ans3 = check.pred(y, x[, c(3, 5, 7)], selected)
ans4 = check.pred(y, x[, c(3, 8, 9)], selected)

allans = matrix(0, nrow = 4, ncol = 2)
colnames(allans) = c("mean", "sd")
allans[1, ] = c(ans1$mean, ans1$sd)
allans[2, ] = c(ans2$mean, ans2$sd)
allans[3, ] = c(ans3$mean, ans3$sd)
allans[4, ] = c(ans4$mean, ans4$sd)
rownames(allans) = c("123", "1to9", "357", "389")

dump(c("selected", "ans1", "ans2", "ans3", "ans4", "allans"), 
       file = "example-4-b.Rdata")


######################################################################
## END OF THE FILE
######################################################################
